==> techne-0.2.1.tar.gz: techne-0.2.1/src/flexible.c <==
/* Copyright (C) 2009 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <lua.h>
#include <lauxlib.h>
#include <GL/gl.h>

#include "fabric.h"

static int generic_tostring(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);
    lua_pushstring(L, [object name]);
   
    return 1;
}

static int generic_index(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);
    
    [object get];
    
    return 1;
}

static int generic_newindex(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);
    
    [object set];

    return 0;
}

static int generic_gc(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);

    [object free];

    return 0;
}

static int fabric_len(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);

    lua_newtable(L);
    
    lua_pushnumber(L, [object vertices]);
    lua_rawseti(L, -2, 1);

    lua_pushnumber(L, [object indices] / 3);
    lua_rawseti(L, -2, 2);

    return 1;
}

static int fabric_call (lua_State *L)
{
    id parent, child;

    parent = *(id *)lua_touserdata(L, 1);

    /* Create the userdata... */
	
    child = [[parent copy] init];
    *(id *)lua_newuserdata(L, sizeof(id)) = child;
    
    lua_newtable (L);
    lua_pushstring(L, "__len");
    lua_pushcfunction(L, fabric_len);
    lua_settable(L, -3);
    lua_pushstring(L, "__index");
    lua_pushcfunction(L, generic_index);
    lua_settable(L, -3);
    lua_pushstring(L, "__newindex");
    lua_pushcfunction(L, generic_newindex);
    lua_settable(L, -3);
    lua_pushstring(L, "__tostring");
    lua_pushcfunction(L, generic_tostring);
    lua_settable(L, -3);
    lua_pushstring(L, "__gc");
    lua_pushcfunction(L, generic_gc);
    lua_settable(L, -3);
    lua_pushstring(L, "__reference");
    lua_pushvalue(L, 1);
    lua_settable(L, -3);
    lua_setmetatable(L, -2);
    
    /* ...and initialize it. */

    if(lua_istable(L, 2)) {
	lua_pushnil(L);
	
	while(lua_next(L, 2)) {
	    lua_pushvalue(L, -2);
	    lua_insert(L, -2);
	    lua_settable(L, 3);
	}
    }

    lua_pushstring (L, "userdata");
    lua_gettable (L, LUA_REGISTRYINDEX);
    lua_pushlightuserdata (L, child);
    lua_pushvalue (L, -3);
    lua_settable (L, -3);
    lua_pop(L, 1);

    return 1;
}

static int fabric_tostring(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);

    lua_pushstring(L, [object name]);
    lua_pushstring(L, " cloth foundry");
    lua_concat(L, 2);
    
    return 1;
}

static int fabric_gc(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);

    [object freeBuffers];
    [object free];

    return 0;
}

static int constructfabric(lua_State *L)
{
    id object;
    float *vertices, *texture;
    unsigned int *indices;
    int *stretch, *shear, *bend, *nodes;
    int i, j, size[6];
    
    luaL_checktype(L, 1, LUA_TTABLE);

    lua_pushstring(L, "size");
    lua_gettable(L, 1);

    if(lua_istable(L, -1)) {
        for(i = 0 ; i < 6 ; i += 1) {
            lua_rawgeti(L, -1, i + 1);
            size[i] = lua_tonumber(L, -1);
                
            lua_pop(L, 1);
        }
    }

    lua_pushstring(L, "vertices");
    lua_gettable(L, 1);
    
    /* Get the vertices. */

    vertices = (float *)malloc(3 * size[0] * sizeof(float));
    texture = (float *)malloc(2 * size[0] * sizeof(float));

    for(i = 0 ; i < size[0] ; i += 1) {
	for (j = 0 ; j < 3 ; j += 1) {
	    lua_pushinteger (_L, 5 * i + j + 1);
	    lua_gettable (_L, -2);

	    vertices[3 * i + j] = (float)lua_tonumber(L, -1);

	    lua_pop(L, 1);
	}
	
	for (j = 0 ; j < 2 ; j += 1) {
	    lua_pushinteger (_L, 5 * i + j + 4);
	    lua_gettable (_L, -2);

	    texture[2 * i + j] = (float)lua_tonumber(L, -1);

	    lua_pop(L, 1);
	}	
    }

    lua_pushstring(L, "stretch");
    lua_gettable(L, 1);

    /* The stretch springs. */

    stretch = (int *)malloc(2 * size[2] * sizeof(int));

    for(i = 0 ; i < 2 * size[2] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	stretch[i] = (int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    lua_pushstring(L, "shear");
    lua_gettable(L, 1);

    /* The shear springs. */

    shear = (int *)malloc(2 * size[3] * sizeof(int));

    for(i = 0 ; i < 2 * size[3] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	shear[i] = (int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    lua_pushstring(L, "bend");
    lua_gettable(L, 1);

    /* The bend springs. */

    bend = (int *)malloc(2 * size[4] * sizeof(int));

    for(i = 0 ; i < 2 * size[4] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	bend[i] = (int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    lua_pushstring(L, "nodes");
    lua_gettable(L, 1);

    /* The nodes. */

    nodes = (int *)malloc(size[5] * sizeof(int));

    for(i = 0 ; i < size[5] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	nodes[i] = (int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    lua_pushstring(L, "indices");
    lua_gettable(L, 1);

    /* And the indices. */

    indices = (unsigned int *)malloc(size[1] * sizeof(unsigned int));
    
    for(i = 0 ; i < size[1] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	indices[i] = (unsigned int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    /* Create and initialize the fabric userdata. */
    
    object = [[Fabric alloc] initWithVertices: vertices
	                        uvcoordinates: texture
	                              indices: indices
			       stretchSprings: stretch
			         shearSprings: shear
			          bendSprings: bend
				     andNodes: nodes
	                               ofSize: size];
    
    *(id *)lua_newuserdata(L, sizeof(id)) = object;

    lua_newtable(L);
    lua_pushstring(L, "__tostring");
    lua_pushcfunction(L, fabric_tostring);
    lua_settable(L, -3);
    lua_pushstring(L, "__call");
    lua_pushcfunction(L, fabric_call);
    lua_settable(L, -3);
    lua_pushstring(L, "__gc");
    lua_pushcfunction(L, fabric_gc);
    lua_settable(L, -3);
    lua_setmetatable(L, -2);

    free(vertices);
    free(texture);
    free(indices);
    free(stretch);
    free(shear);
    free(bend);
    free(nodes);
    
    return 1;
}

int luaopen_flexible (lua_State *L)
{
    const luaL_Reg meshes[] = {
	{"fabric", constructfabric},
	{NULL, NULL}
    };

    luaL_register (L, lua_tostring (L, 1), meshes);

    return 0;
}

==> techne-0.2.3.tar.gz: techne-0.2.3/src/flexible.c <==
/* Copyright (C) 2009 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <lua.h>
#include <lauxlib.h>
#include <GL/gl.h>

#include "fabric.h"

static int generic_tostring(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);
    lua_pushstring(L, [object name]);
   
    return 1;
}

static int generic_index(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);
    
    [object get];
    
    return 1;
}

static int generic_newindex(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);
    
    [object set];

    return 0;
}

static int generic_gc(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);

    [object free];

    return 0;
}

static int dummy_index(lua_State *L)
{
    lua_pushnil (L);
    
    return 1;
}

static int dummy_newindex(lua_State *L)
{
    return 0;
}

static int fabric_len(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);

    lua_newtable(L);
    
    lua_pushnumber(L, [object vertices]);
    lua_rawseti(L, -2, 1);

    lua_pushnumber(L, [object indices] / 3);
    lua_rawseti(L, -2, 2);

    return 1;
}

static int fabric_call (lua_State *L)
{
    id parent, child;

    parent = *(id *)lua_touserdata(L, 1);

    /* Create the userdata... */
	
    child = [[parent copy] init];
    *(id *)lua_newuserdata(L, sizeof(id)) = child;
    
    lua_newtable (L);
    lua_pushstring(L, "__len");
    lua_pushcfunction(L, fabric_len);
    lua_settable(L, -3);
    lua_pushstring(L, "__index");
    lua_pushcfunction(L, generic_index);
    lua_settable(L, -3);
    lua_pushstring(L, "__newindex");
    lua_pushcfunction(L, generic_newindex);
    lua_settable(L, -3);
    lua_pushstring(L, "__tostring");
    lua_pushcfunction(L, generic_tostring);
    lua_settable(L, -3);
    lua_pushstring(L, "__gc");
    lua_pushcfunction(L, generic_gc);
    lua_settable(L, -3);
    lua_pushstring(L, "__reference");
    lua_pushvalue(L, 1);
    lua_settable(L, -3);
    lua_setmetatable(L, -2);
    
    /* ...and initialize it. */

    if(lua_istable(L, 2)) {
	lua_pushnil(L);
	
	while(lua_next(L, 2)) {
	    lua_pushvalue(L, -2);
	    lua_insert(L, -2);
	    lua_settable(L, 3);
	}
    }

    lua_pushstring (L, "userdata");
    lua_gettable (L, LUA_REGISTRYINDEX);
    lua_pushlightuserdata (L, child);
    lua_pushvalue (L, -3);
    lua_settable (L, -3);
    lua_pop(L, 1);

    return 1;
}

static int fabric_tostring(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);

    lua_pushstring(L, [object name]);
    lua_pushstring(L, " foundry");
    lua_concat(L, 2);
    
    return 1;
}

static int fabric_gc(lua_State *L)
{
    id object;

    object = *(id *)lua_touserdata(L, 1);

    [object freeBuffers];
    [object free];

    return 0;
}

static int constructfabric(lua_State *L)
{
    id object;
    float *vertices, *texture;
    unsigned int *indices;
    int *stretch, *shear, *bend, *nodes;
    int i, j, size[6];
    
    luaL_checktype(L, 1, LUA_TTABLE);

    lua_pushstring(L, "size");
    lua_gettable(L, 1);

    if(lua_istable(L, -1)) {
        for(i = 0 ; i < 6 ; i += 1) {
            lua_rawgeti(L, -1, i + 1);
            size[i] = lua_tonumber(L, -1);
                
            lua_pop(L, 1);
        }
    }

    lua_pushstring(L, "vertices");
    lua_gettable(L, 1);
    
    /* Get the vertices. */

    vertices = (float *)malloc(3 * size[0] * sizeof(float));
    texture = (float *)malloc(2 * size[0] * sizeof(float));

    for(i = 0 ; i < size[0] ; i += 1) {
	for (j = 0 ; j < 3 ; j += 1) {
	    lua_pushinteger (_L, 5 * i + j + 1);
	    lua_gettable (_L, -2);

	    vertices[3 * i + j] = (float)lua_tonumber(L, -1);

	    lua_pop(L, 1);
	}
	
	for (j = 0 ; j < 2 ; j += 1) {
	    lua_pushinteger (_L, 5 * i + j + 4);
	    lua_gettable (_L, -2);

	    texture[2 * i + j] = (float)lua_tonumber(L, -1);

	    lua_pop(L, 1);
	}	
    }

    lua_pushstring(L, "stretch");
    lua_gettable(L, 1);

    /* The stretch springs. */

    stretch = (int *)malloc(2 * size[2] * sizeof(int));

    for(i = 0 ; i < 2 * size[2] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	stretch[i] = (int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    lua_pushstring(L, "shear");
    lua_gettable(L, 1);

    /* The shear springs. */

    shear = (int *)malloc(2 * size[3] * sizeof(int));

    for(i = 0 ; i < 2 * size[3] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	shear[i] = (int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    lua_pushstring(L, "bend");
    lua_gettable(L, 1);

    /* The bend springs. */

    bend = (int *)malloc(2 * size[4] * sizeof(int));

    for(i = 0 ; i < 2 * size[4] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	bend[i] = (int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    lua_pushstring(L, "nodes");
    lua_gettable(L, 1);

    /* The nodes. */

    nodes = (int *)malloc(size[5] * sizeof(int));

    for(i = 0 ; i < size[5] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	nodes[i] = (int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    lua_pushstring(L, "indices");
    lua_gettable(L, 1);

    /* And the indices. */

    indices = (unsigned int *)malloc(size[1] * sizeof(unsigned int));
    
    for(i = 0 ; i < size[1] ; i += 1) {
	lua_pushinteger (_L, i + 1);
	lua_gettable (_L, -2);

	indices[i] = (unsigned int)lua_tonumber(L, -1);

	lua_pop(L, 1);
    }

    /* Create and initialize the fabric userdata. */
    
    object = [[Fabric alloc] initWithVertices: vertices
	                        uvcoordinates: texture
	                              indices: indices
			       stretchSprings: stretch
			         shearSprings: shear
			          bendSprings: bend
				     andNodes: nodes
	                               ofSize: size];
    
    *(id *)lua_newuserdata(L, sizeof(id)) = object;

    lua_newtable(L);
    lua_pushstring(L, "__tostring");
    lua_pushcfunction(L, fabric_tostring);
    lua_settable(L, -3);
    lua_pushstring(L, "__call");
    lua_pushcfunction(L, fabric_call);
    lua_settable(L, -3);
    lua_pushstring(L, "__gc");
    lua_pushcfunction(L, fabric_gc);
    lua_settable(L, -3);
    lua_pushstring(L, "__index");
    lua_pushcfunction(L, dummy_index);
    lua_settable(L, -3);
    lua_pushstring(L, "__newindex");
    lua_pushcfunction(L, dummy_newindex);
    lua_settable(L, -3);
    lua_setmetatable(L, -2);

    lua_pushstring (L, "userdata");
    lua_gettable (L, LUA_REGISTRYINDEX);
    lua_pushlightuserdata (L, object);
    lua_pushvalue (L, -3);
    lua_settable (L, -3);
    lua_pop(L, 1);

    free(vertices);
    free(texture);
    free(indices);
    free(stretch);
    free(shear);
    free(bend);
    free(nodes);
    
    return 1;
}

int luaopen_flexible (lua_State *L)
{
    const luaL_Reg meshes[] = {
	{"fabric", constructfabric},
	{NULL, NULL}
    };

    luaL_register (L, lua_tostring (L, 1), meshes);

    return 0;
}
